<?php

namespace Drupal\denormalizer\Plugin\QueueWorker;

use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Entity\EntityInterface;

/**
 * Processes Denormalizer queue tasks.
 *
 * @QueueWorker(
 *   id = "denormalizer_queue",
 *   title = @Translation("Denormalizer Queue Worker"),
 *   cron = {"time" = 60}
 * )
 */
class DenormalizerWorker extends QueueWorkerBase {

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    \Drupal::logger('denormalizer')->info('Processing queue item: @data', ['@data' => json_encode($data)]);

    // Extract data from the queue item.
    $denormalizer_table_id = $data['denormalizer_table_id'];
    $entity_type_id = $data['entity_type_id'];
    $entity_id = $data['entity_ids'];
    $is_new = $data['is_new'];
    $delete = $data['delete'];

    // Load the denormalizer table entity if an ID is provided.
    /** @var \Drupal\denormalizer\Entity\DenormalizerTableInterface|null $denormalizer_table */
    $denormalizer_table = \Drupal::entityTypeManager()->getStorage('denormalizer_table')->load($denormalizer_table_id);

    $entity_storage = \Drupal::entityTypeManager()->getStorage($entity_type_id);
    $entity = $entity_storage->load($entity_id);

    if (!$entity instanceof EntityInterface) {
      \Drupal::logger('denormalizer')->error('Entity not found for entity ID: @id', ['@id' => $entity_id]);
      return;
    }
    if ($delete) {
      // Handle entity deletion.
      \Drupal::logger('denormalizer')->info('Deleting entity data from denormalizer table for entity ID: @id', ['@id' => $entity->id()]);
      $denormalizer_table->deleteDataFromEntityIntoDbTable($entity);
    }
    elseif ($is_new) {
      // Handle entity creation.
      \Drupal::logger('denormalizer')->info('Inserting new entity data into denormalizer table for entity ID: @id', ['@id' => $entity->id()]);
      $denormalizer_table->insertDataFromEntityIntoDbTable($entity);
    }
    else {
      // Handle entity update or bulk processing.
      \Drupal::logger('denormalizer')->info('Updating entity data in denormalizer table for entity ID: @id', ['@id' => $entity->id()]);
      $denormalizer_table->updateDataFromEntityIntoDbTable($entity);
    }

    \Drupal::logger('denormalizer')->info('Queue item processed successfully.');
  }

}
